% Benchmarks
:- module(bench, [
	bench01/1,
	bench02/1,
	bench03/1,
	bench04/1,
	bench05/1,
	bench06/1,
	bench07/1,
	bench08/1
]).

:- use_module(isa/alu).
:- use_module(isa/control).


% Bench 01
% V2 := 11 + 23
bench01([
    const(0, 11),
    const(1, 23),
    addt(2, 1, 0),
    stop
]).

% Bench 02
% Increment the vector by 1
bench02([
    load(0, tid),
    const(1, 1),
    addt(2, 0, 1),
    store(2, tid),
    stop
]).

% Bench 03
% if m[tid] >= 7 then m[tid] += 1
bench03([
    load(0, tid),    % 1:  v0 = m[tid]
    const(1, 7),     % 2:  v1 = 7
    lth(1, 0, 1),    % 3:  v1 = v0 < v1 ? 1 : 0
    branch(1, 7),    % 4:  if v1 != 0 then goto 7
    const(2, 1),     % 5:  v2 = 1
    addt(0, 0, 2),   % 6:  v0 = v0 + v2
    sync,            % 7:  sync
    store(0, tid),   % 8:  m[tid] = v0
    stop             % 9:
]).

% Bench 04
% if (m[tid] < 0 then m[tid] += 10 else m[tid] -= 10)
bench04([
    load(0, tid),    % 1:  v0 = m[tid]
    const(1, 0),     % 2:  v1 = 0
    lth(1, 0, 1),    % 3:  v1 = v0 < v1 ? 1 : 0
    branch(1, 8),    % 4:  if v1 != 0 then goto L
    const(2, -10),   % 5:  v2 = -10
    addt(0, 0, 2),   % 6:  v0 = v0 + v2
    jump(10),        % 7:  goto 10
    const(2, 10),    % 8:  v2 = 10
    addt(0, 0, 2),   % 9:  v0 = v0 + v2
    sync,            % 10: sync
    store(0, tid),   % 11: m[tid] = v0
    stop             %
]).

% Bench 05
/* while (m[tid] > 0) m[tid]-- */
bench05([
    load(0, tid),    % 1:  v0 = m[tid]
    const(1, 0),     % 2:  v1 = 0
    lth(2, 0, 1),    % 3:  v2 = v0 < v1 ? 1 : 0
    branch(2, 8),    % 4:  if v2 != 0 then goto 8
    const(3, -1),    % 5:  v3 = -1
    addt(0, 0, 3),   % 6:  v0 = v0 + v3
    jump(3),         % 7:  jump 3
    sync,            % 8:  sync
    store(0, tid),   % 9:  m[tid] = v0
    stop             % 10:
]).

% Bench 06
/*
   while (m[tid] > 0) {
     if (5 < m[tid]) {
       m[tid] -= 5
     } else {
       m[tid] -= 3
     }
     m[tid]++
   }
 */
bench06([
    load(0, tid),    % 1:  v0 = m[tid]
    const(1, 0),     % 2:  v1 = 0
    lth(1, 0, 1),    % 3:  v1 = v0 < v1 ? 1 : 0
    branch(1, 17),   % 4:  if v1 == 1 then goto 17
    const(1, 5),     % 5:  v1 = 5
    lth(1, 1, 0),    % 6:  v1 = v1 < v0 ? 1 : 0
    branch(1, 11),   % 7:  if v1 == 1 then goto 11
    const(1, -3),    % 8:  v1 = -3
    addt(0, 0, 1),   % 9:  v0 = v0 + v1
    jump(13),        % 10: jump 13
    const(1, -5),    % 11: v1 = -5
    addt(0, 0, 1),   % 12: v0 = v0 + v1
    sync,            % 13: sync
    const(1, 1),     % 14: v1 = 1
    addt(0, 0, 1),   % 15: v0 = v0 + v1
    jump(2),         % 16: jump 2
    sync,            % 17: sync
    store(0, tid),   % 18: m[tid] = v0
    stop             % 19: stop
]).

% Bench 07
% v1 := atomic{++m[0]}
bench07([
    const(0, 0),
    atominc(1, 0),
    stop
]).

% Bench 08
% Everywhere block: inline operations
bench08([
	load(0, tid),		%  1: v0 = m[tid]
	const(1, 1),		%  2: v1 = 1
	lth(1, 0, 1),		%  3: v1 = v0 < v1 ? 1 : 0
	branch(1, 18),		%  4: if v1 == 1: goto 14
	const(1, 2),		%  5: v1 = 2
	lth(1, 0, 1),		%  6: v1 = v0 < v1 ? 1 : 0
	branch(1, 17),		%  7: if v1 == 1: goto 14
	const(1, 3),		%  8: v1 = 3
	lth(1, 0, 1),		%  9: v1 = v0 < v1 ? 1 : 0
	branch(1, 16),		% 10: if v1 == 1: goto 14
	everywhere,			% 11: everywhere begin
						%     save caller / registers
	load(1, tid),		% 12: v1 = m[tid]
	load(2, tid),		% 13: v2 = m[tid]
	addt(0, 1, 2),		% 14: v0 = v1 + v2
	end_everywhere,		% 15: everywhere end
						%     restores caller / registers
	sync,				% 16: sync
	sync,				% 17: sync
	sync,				% 18: sync
	stop				% 19: stop
]).

% Bench 09
% Everywhere block: kernel call
bench09([
	load(0, tid),		%  1: v0 = m[tid]
	const(1, 1),		%  2: v1 = 1
	lth(1, 0, 1),		%  3: v1 = v0 < v1 ? 1 : 0
	branch(1, 18),		%  4: if v1 == 1: goto 14
	const(1, 2),		%  5: v1 = 2
	lth(1, 0, 1),		%  6: v1 = v0 < v1 ? 1 : 0
	branch(1, 17),		%  7: if v1 == 1: goto 14
	const(1, 3),		%  8: v1 = 3
	lth(1, 0, 1),		%  9: v1 = v0 < v1 ? 1 : 0
	branch(1, 16),		% 10: if v1 == 1: goto 14
	everywhere,			% 11: everywhere begin
						%     save caller / registers
	kernel(0),			% 12: call kernel 0
	end_everywhere,		% 13: everywhere end
						%     restores caller / registers
	sync,				% 14: sync
	sync,				% 15: sync
	sync,				% 16: sync
	stop				% 17: stop
]).
