% Debug utilities
:- module(debug, [

	% debug(PC, Instr, Threads, LocalMemBank, DivergenceStack,
	%	ContextStack, SharedMemory)
	debug/7,

	% Prints out the given error
	error/3

]).


padding('\n      ').

% Writes the sequence of data
cout([]).
cout([H|T]) :- write(H), cout(T).
cout(E) :- write(E).

% Prints out the given program counter (PC)
pad(Value, 0) :- write(Value).
pad(Value, 1) :- write(Value).
pad(Value, Length) :-
	Tmp is Length - 1,
	Base is 10 ** Tmp,
	Quot is Value // Base,
	(Quot > 0 -> write(Value) ; write(0), pad(Value, Tmp)).

% Prints out the Memory contents
debugMemory([]).
debugMemory([(Index, Value)]) :- cout(['#', Index, ': ', Value]).
debugMemory([(Index, Value)|T]) :-
	cout(['#', Index, ': ', Value, ', ']), debugMemory(T).

% Prints out an instruction
debugInstr(PC, Instr) :- pad(PC, 5), cout([' ', Instr]).

% Prints out the shared memory contencts
debugShdMem(SharedMem) :-
	padding(Padding), cout([Padding, '       SharedMem']),
	cout([Padding, '       {']), debugMemory(SharedMem), cout('}').

% Prints out the contents of the Threads
debugThreads(Threads, LocalMemBank) :-
	padding(Padding), cout([Padding, 'TID', '    ', 'LocalMem']),
	debugThreads_(Threads, LocalMemBank, 0).

debugThreads_(_, [], _).

debugThreads_(Threads, [LM|LMB], Index) :-
	padding(Padding), cout([Padding, Index, '      ', '{']),
	debugMemory(LM), cout('}'),
	(memberchk(Index, Threads)
	->	cout('    ACTIVE')
	;	cout('    INACTIVE')),
	NewIndex is Index + 1,
	debugThreads_(Threads, LMB, NewIndex).

% Prints out the divergence stack
debugDivStack(DivStack) :-
	padding(Padding),
	cout([Padding, 'Depth     Next      Label     Active/Inactive']),
	(DivStack == []
	->	cout([Padding, '--        --        --        -- / --'])
	;	debugDivStack_(DivStack)).
debugDivStack_([]).
debugDivStack_([(PC, Active, Label, Inactive)|T]) :-
	length(T, Depth),
	padding(Padding), cout(Padding),
	pad(Depth, 5), cout('     '),
	pad(PC, 5), cout('     '),
	pad(Label, 5), cout('     '),
	cout([Active, ' / ', Inactive]),
	debugDivStack_(T).

% Prints out the context stack
debugConStack(ContextStack) :-
	padding(Padding), cout([Padding, 'Context Stack']),
	(_, ConStack) = ContextStack,
	(ConStack == []
	->	cout([Padding, '--'])
	;	debugConStack_(ConStack)).

debugConStack_([]).
debugConStack_([(_, _, PC)|ConStack]) :-
	padding(Padding), cout(Padding), pad(PC, 5),
	debugConStack_(ConStack).

% Prints out the current state of the machine
debug(PC, Instr, Threads, LocalMemBank, DivStack, ConStack, ShdMem) :-
	cout('\n\n'),
	debugInstr(PC, Instr),
	debugShdMem(ShdMem),
	debugThreads(Threads, LocalMemBank),
	debugConStack(ConStack),
	debugDivStack(DivStack).

% Prints out the given error
error(PC, Instr, Error) :-
	cout('\n\n\n'), padding(Padding),
	pad(PC, 5), cout([' ', Instr, Padding, 'Error: ', Error]),
	halt.
