% Thread divergence
:- module(divergence, [

	% splitThreads([ThreadID], LocalMemoryBank, ConditionVar, Θz, Θn)
	divergeThreads/5,

	% updateThreadStack(DivStack, DivergentThreads, PC, Label, NewDivStack)
	updateDivStack/5

]).

:- use_module(utility).
:- use_module(isa/memory).


% Splits divergent threads into two sets, Θz and Θn,
% using as filter the value of a variable V, local to each thread
divergeThreads([], _, _, [], []).

divergeThreads([Tid|Ts], LMB, V, [Tid|Zs], Ns) :-
	getval(Tid, LMB, V, 0),
	divergeThreads(Ts, LMB, V, Zs, Ns).

divergeThreads([Tid|Ts], LMB, V, Zs, [Tid|Ns]) :-
	getval(Tid, LMB, V, N),
	N \= 0,
	divergeThreads(Ts, LMB, V, Zs, Ns).

% Updates the synchronization stack of divergent threads
% updateDivStack(DivStack, DivergentThreads, PC, Label, NewDivStack)
updateDivStack([], DivIDs, PC, L, [(PC, [], L, DivIDs)]).

updateDivStack([(PCC, [], LL, NTT)|ST], NT, PC, L, NewST) :-
	PC \= PCC,
	NewST = [(PC, [], L, NT), (PCC, [], LL, NTT)|ST].

updateDivStack([(PC, [], L, NTT)|ST], NT, PC, L, [(PC, [], L, NTTT)|ST]) :-
	union(NT, NTT, NTTT).
