% Memory management
:- module(memory, [

	% getval(LocalMemory, Variable, Value)
	% getval(SharedMemory, Variable, Value)
	getval/3,

	% getval(ThreadID, LocalMemoryBank, Variable, Value)
	getval/4,

	% lookup(Program, Index, Instruction)
	lookup/3,

	% setval(SharedMemory, Addr, Value, NewSharedMemory)
	setval/4,

	% setval(ThreadID, LocalMemoryBank, Variable, Value, NewLocalMemoryBank)
	setval/5

]).


% Maps variables into registers
register(tid).
register(R) :- integer(R).

% Locates the local memory of a specific thread ID in the local memory bank
getLocalMem(_, [], _) :-
	write('\n-- Invalid TID or local memory bank size on read!'), halt.
getLocalMem(0, [LM|_], LM).
getLocalMem(Tid, [_|LMs], LM) :-
	Tid > 0, NewTid is Tid - 1,
	getLocalMem(NewTid, LMs, LM).

% Updates the local memory of a specific thread ID in the local memory bank
updateLocalMem(_, _, [], _) :-
	write('\n-- Invalid TID or local memory bank size on write!'), halt.
updateLocalMem(0, NewLM, [_|LMBank], [NewLM|LMBank]).
updateLocalMem(Tid, NewLM, [LM|LMBank], [LM|NewLMBank]) :-
	Tid > 0, NewTid is Tid - 1,
	updateLocalMem(NewTid, NewLM, LMBank, NewLMBank).

% Retrieves the value from shared or specific local memory
getval([(V, N)|_], V, N).

getval([(X, _)|LM], V, N) :-
	X \= V, getval(LM, V, N).

% Retrieves the thread identifier
getval(Tid, _, tid, Tid) :- !.

% Retrieves value from the local memory of thread Tid
getval(Tid, LMBank, V, N) :-
	getLocalMem(Tid, LMBank, LM),
	getval(LM, V, N).

% Retrieves the N-th instruction from the input program
lookup([Inst|_], 1, Inst).
lookup([_|Insts], N, Inst) :-
	N > 1, M is N - 1, lookup(Insts, M, Inst).

% Sets the value of local variable V to N
setval(Tid, LMBank, V, N, NewLMBank) :-
	getLocalMem(Tid, LMBank, LM),
	setval(LM, V, N, NewLM),
	updateLocalMem(Tid, NewLM, LMBank, NewLMBank).

setval([(V, _)|LM], V, N, [(V, N)|LM]).
setval([(X, XValue)|LM], V, N, [(X, XValue)|NLM]) :-
	X \= V, setval(LM, V, N, NLM).
